part of dartemporal;

/// The Is TE provides funcationality to test for occurrences after a deadline of the various DateTime components such as Year, Month, etc.
class TEInRange extends TExpression {
  DateTime _first;  // The first DateTime that matches this TE
  DateTime _last;   // The last DateTime that matches this TE (inclusive)
  TEIsAfter  _start;   // Holds the left boundry object
  TEIsBefore _end;  // Holds the right boundry object

  /// Returns the DateTime on which this TE starts matching
  DateTime first() {
    return _first;
  }

  /// Returns the DateTime on which this TE stops matching
  DateTime last() {
    return _last;
  }


  /// Allow this class to be "stringified", this does not return the JSON string but does return a Map that can be stringified.
  Map toJson() {
    // Members to JSON list

    return({"TEMasterType": "TEInRange", "ValueAfter":_start.toJson(), "ValueBefore":_end.toJson()});
  }

  /// Create a TEIs object from a Map object that was created manually (why?) or from the toJson method
  ///   is a class factory
  TEInRange.fromJson(Map source) {
    if(source["TEMasterType"] != "TEInRange") {
      throw new InvalidJSONException("Invalid JSON Format: Not TEInRange");
    }

    _start = new TEIsAfter.fromJson(source["ValueAfter"]);
    _end = new TEIsBefore.fromJson(source["ValueBefore"]);
    _first = _start.first();
    _last = _end.last();
  }

  /// Constructor to match all DateTimes between two given years.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Year(int startYear, int endYear, [bool startInclusive = true, bool endInclusive = true]) {
    _start = new TEIsAfter.Year(startYear, startInclusive);
    _end = new TEIsBefore.Year(endYear, endInclusive);
    _first =_start.first();
    _last = _end.last();
  }

  /// Constructor to match all DateTimes that occur between two given months, regardless of year.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Month(int startMonth, int endMonth, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;        // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.Month(startMonth, startInclusive);
    _end = new TEIsBefore.Month(endMonth, endInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific days of the week (Monday for example) regardless of year
  /// Uses the DateTime constants (DateTime.MONDAY) to select
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.DayOfWeek(int startDOW, int endDOW, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;       // first and last are null because year, month, and week are undefined
    _last  = null;
    _start = new TEIsAfter.DayOfWeek(startDOW, startInclusive);
    _end = new TEIsBefore.DayOfWeek(endDOW, endInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific days of every month (3rd of every month for example) regardless of year
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.DayOfMonth(int startDOM, int endDOM, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;      // first and last are null because year and month are undefined
    _last = null;
    _start = new TEIsAfter.DayOfMonth(startDOM, startInclusive);
    _end = new TEIsBefore.DayOfMonth(endDOM, endInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific days of every year (day 212 of every year)
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.DayOfYear(int startDOY, int endDOY, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;     // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.DayOfYear(startDOY, startInclusive);
    _end = new TEIsBefore.DayOfYear(endDOY, endInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific hours of every day.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Hour(int startHour, int endHour, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;    // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.Hour(startHour, startInclusive);
    _end = new TEIsBefore.Hour(endHour, endInclusive);
  }

  /// Constructor to match all DateTimes that occur during two specific minutes of every hour;
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Minute(int startMinute, int endMinute, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;    // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.Minute(startMinute, startInclusive);
    _end = new TEIsBefore.Minute(endMinute, endInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific seconds of every minute;
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Second(int startSecond, int endSecond, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;    // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.Second(startSecond, startInclusive);
    _end = new TEIsBefore.Second(endSecond, startInclusive);
  }

  /// Constructor to match all DateTimes that occur between two specific milliseconds of every second;
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Millisecond(int startMillisecond, int endMillisecond, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;    // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.Millisecond(startMillisecond, startInclusive);
    _end = new TEIsBefore.Millisecond(endMillisecond, startInclusive);
  }

  /// Constructor to match all dateTimes that occur during two specific weeks of the year.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.WeekOfYear(int startWOY, int endWOY, [bool startInclusive = true, bool endInclusive = true]) {
    _first = null;    // first and last are null because year is undefined
    _last = null;
    _start = new TEIsAfter.WeekOfYear(startWOY, startInclusive);
    _end = new TEIsBefore.WeekOfYear(endWOY, endInclusive);
  }

  /// Constructor to match all DateTimes that occur after a given DateTime
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.DateTime(DateTime startDate, DateTime endDate, [bool startInclusive = true, bool endInclusive = true]) {
    _start = new TEIsAfter.DateTime(startDate, startInclusive);
    _end = new TEIsBefore.DateTime(endDate, endInclusive);
    _first = _start.first();
    _last = _end.last();
  }

  /// Constructor to match all DateTimes that occur after a given Date, ignoring any time component of the start and end DateTimes provided.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.Date(DateTime startDate, DateTime endDate, [bool startInclusive = true, bool endInclusive = true]) {
    _start = new TEIsAfter.Date(startDate, startInclusive);
    _end = new TEIsBefore.Date(endDate, endInclusive);
    _first = _start.first();
    _last = _end.last();
  }

  /// Constructor to match all DateTimes that occur after a given Date, ignoring any time component of the start and end DateTimes provided.
  /// By default this is inclusive, but that can be changed with the boolean parameters.
  TEInRange.WeekOfMonth(int startDate, int endDate) {
    _start = new TEIsAfter.WeekOfMonth(startDate);
    _end = new TEIsBefore.WeekOfMonth(endDate);
    _first = _start.first();
    _last = _end.last();
  }

  /// Test to see if an arbitrary testDate is a match for this TE
  bool includes(DateTime testDate) {
    return _start.includes(testDate) && _end.includes(testDate);
  }

}