part of dartemporal;


/// Represents a logical AND of all member TE's
class TEIntersect extends TExpression {
  List<TExpression> _members = [];  // Holds the TE's to be UNIONed

  List<TExpression> get members => _members;

  /// Class constructor
  TEIntersect(List<TExpression> members) {
    _members = members;
  }

  bool includes(DateTime date) {
    bool returnValue = true;

    _members.forEach((c){
      if(c.includes(date) == false) {
       returnValue = false;
      }
    });

    return returnValue;
  }

  /// Allow this class to be "stringified", this does not return the JSON string but does return a Map that can be stringified.
  Map toJson() {
    // Members to JSON list
    List<Map> memberJSON;
    _members.forEach((c) {
      memberJSON.add(c.toJson());
    });

    return({"TEMasterType": "TEIntersect", "Members": memberJSON});
  }


  /// Create a TEIs object from a Map object that was created manually (why?) or from the toJson method
  TEIntersect.fromJson(Map source) {
    List<TExpression> members = [];

    if(source["TEMasterType"] != "TEIntersect") {
      throw new InvalidJSONException("Invalid JSON Format: Not TEIntersect");
    }

    source["Members"].forEach((c){
      _members.add(new TExpression.fromJson(c));
    });
  }

  // As this is an INTERSECT, the LATEST start date defines the first.
  //   Will return null if there is no EARLIEST date
  DateTime first() {
    DateTime first = null;

    _members.forEach((c) {
      DateTime testDate = c.first();

      if(testDate != null) {
        if(first == null) {
          first = testDate;
        } else {
          if(first.isBefore(testDate)) {
            first = testDate;
          }
        }
      }
    });

    return first;
  }

  // As this is a INTERSECT, the LATEST start date defines the last.
  //   Will return null if there is no LATEST date
  DateTime last() {
    DateTime last = null;

    _members.forEach((c) {
      DateTime testDate = c.last();

      if(testDate != null) {
        if(last == null) {
          last = testDate;
        } else {
          if(last.isAfter(testDate)) {
            last = testDate;
          }
        }
      }
    });

    return last;
  }
}