part of dartemporal;


/// Represents a logical OR of all member TE's
class TEUnion extends TExpression {
  List<TExpression> _members;  // Holds the TE's to be UNIONed

  List<TExpression> get members => _members;

  /// Class constructor
  TEUnion(List<TExpression> members) {
    _members = members;
  }

  bool includes(DateTime date) {
    bool returnValue = false;

    _members.forEach((c){
      if(c.includes(date) == true) {
       returnValue = true;
      }
    });

    return returnValue;
  }

  // As this is a UNION, the EARLIEST start date defines the first.
  //   Will return null if there is no EARLIEST date
  DateTime first() {
    bool dateSet = false;
    DateTime first = null;

    _members.forEach((c) {
      DateTime testDate = c.first();

      // If the first date of the TE being tested returns a null first date, then it wins by default
      if(testDate ==  null) {
        dateSet = true;
        first = null;
      } else {
        // This is called if the TE first date is not null, now we need to test if it is earlier
        if(dateSet == false) {
          dateSet = true;
          first = testDate;
        } else {
          // KNOWNS:
          //   dateSet == true
          //   first == a date or null
          //   testDate == a date
          if(first != null) {
            if(first.isAfter(testDate)) {
              first = testDate;
            }
          }
        }
      }
    });

    return first;
  }

  // As this is a UNION, the LATEST start date defines the last.
  //   Will return null if there is no LATEST date
  DateTime last() {
    bool dateSet = false;
    DateTime last = null;

    _members.forEach((c) {
      DateTime testDate = c.last();

      // If the last date of the TE being tested returns a null last date, then it wins by default
      if(testDate ==  null) {
        dateSet = true;
        last = null;
      } else {
        // This is called if the TE last date is not null, now we need to test if it is later
        if(dateSet == false) {
          dateSet = true;
          last = testDate;
        } else {
          // KNOWNS:
          //   dateSet == true
          //   last == a date or null
          //   testDate == a date
          if(last != null) {
            if(last.isBefore(testDate)) {
              last = testDate;
            }
          }
        }
      }
    });

    return last;
  }


  /// Allow this class to be "stringified", this does not return the JSON string but does return a Map that can be stringified.
  Map toJson() {
    // Members to JSON list
    List jsonMembers = [];

    _members.forEach((m) {
      jsonMembers.add(m.toJson());
    });

    return({"TEMasterType": "TEUnion", "Value": jsonMembers});
  }

  /// Create a TEIs object from a Map object that was created manually (why?) or from the toJson method
  ///   is a class factory
  factory TEUnion.fromJson(Map source) {
    List<TExpression> members = [];

    if(source["TEMasterType"] != "TEUnion") {
      throw new InvalidJSONException("Invalid JSON Format: Not TEUnion");
    }

    source["Value"].forEach((m) {
      members.add(new TExpression.fromJson(m));
    });

    return new TEUnion(members);
  }
}