library testCompounding;
import "package:unittest/unittest.dart";
import "package:dartemporal/dartemporal.dart";
import "dart:convert";

testCompounding() {
  test('TE: Diff', () {
    DateTime may_first_2012    = new DateTime.utc(2012, 5,  1, 0, 0, 0, 0);
    DateTime june_first_2012   = new DateTime.utc(2012, 6,  1, 0, 0, 0, 0);
    DateTime june_mid_2012     = new DateTime.utc(2012, 6, 15, 10, 11, 12, 13);
    DateTime june_last_2012    = new DateTime.utc(2012, 7, 1, 0, 0, 0, -1);
    DateTime july_last_2012    = new DateTime.utc(2012, 8, 1, 0, 0, 0, -1);

    // Covers all of June 2012 except the last ms
    TEInRange trueRange = new TEInRange.DateTime(june_first_2012, june_last_2012, true, true);
    TEIs falseRange1 = new TEIs.DateTime(june_mid_2012);
    TEDiff testRange = new TEDiff(trueRange, falseRange1);
    TEDiff testRangejson = new TEDiff.fromJson(JSON.decode(JSON.encode(testRange)));

    expect(testRange.includes(june_first_2012), isTrue);
    expect(testRange.includes(june_last_2012), isTrue);
    expect(testRange.includes(june_mid_2012), isFalse);
    expect(testRange.includes(may_first_2012), isFalse);
    expect(testRange.includes(july_last_2012), isFalse);
    expect(testRangejson.includes(june_first_2012), isTrue);
    expect(testRangejson.includes(june_last_2012), isTrue);
    expect(testRangejson.includes(june_mid_2012), isFalse);
    expect(testRangejson.includes(may_first_2012), isFalse);
    expect(testRangejson.includes(july_last_2012), isFalse);
  });

  test('TE: Union Tests', () {
    DateTime may_first_2012    = new DateTime.utc(2012, 5,  1, 0, 0, 0, 0);
    DateTime june_first_2012   = new DateTime.utc(2012, 6,  1, 0, 0, 0, 0);
    DateTime june_mid_2012     = new DateTime.utc(2012, 6, 15, 10, 11, 12, 13);
    DateTime june_last_2012    = new DateTime.utc(2012, 7, 1, 0, 0, 0, -1);
    DateTime july_last_2012    = new DateTime.utc(2012, 8, 1, 0, 0, 0, -1);

    TEIsBefore left = new TEIsBefore.DateTime(june_first_2012, true);
    TEIsAfter right = new TEIsAfter.DateTime(june_last_2012, true);

    // Everything but June 2012
    TEUnion union = new TEUnion([left, right]);
    TEUnion unionjson = new TEUnion.fromJson(JSON.decode(JSON.encode(union.toJson())));

    // Test first/last
    expect(union.first() == null, isTrue);
    expect(union.last() == null, isTrue);
    expect(unionjson.first() == null, isTrue);
    expect(unionjson.last() == null, isTrue);

    // Test inclusiveness
    expect(unionjson.includes(june_mid_2012), isFalse);
    expect(unionjson.includes(june_first_2012), isTrue);
    expect(unionjson.includes(june_last_2012), isTrue);
    expect(unionjson.includes(may_first_2012), isTrue);
    expect(unionjson.includes(july_last_2012), isTrue);
  });

  test('TE: Not Tests', () {
    TEIs december = new TEIs.Month(12);
    TEIsAfter afterNoon = new TEIsAfter.Hour(12, true);

    DateTime novLast = new DateTime(2013,12,1,0,0,0,-1);  // In Nov after Noon
    DateTime decFirst = new DateTime(2016,12,1,12,0,0,0); // In Dec after Noon
    DateTime decLast = new DateTime(1955,12,31,11,59,59,999); // In Dec before Noon
    DateTime janFirst = new DateTime(-10, 1, 1, 5,0,5,123); // In Jan before Noon

    TENot notDecember = new TENot(december);
    TENot notDecemberjson = new TENot.fromJson(JSON.decode(JSON.encode(notDecember.toJson())));
    TENot notAfternoon = new TENot(afterNoon);
    TENot notAfternoonjson = new TENot.fromJson(JSON.decode(JSON.encode(notAfternoon.toJson())));

    expect(notDecember.includes(novLast), isTrue);
    expect(notDecember.includes(decFirst), isFalse);
    expect(notDecember.includes(decLast), isFalse);
    expect(notDecember.includes(janFirst), isTrue);
    expect(notAfternoon.includes(novLast), isFalse);
    expect(notAfternoon.includes(decFirst), isFalse);
    expect(notAfternoon.includes(decLast), isTrue);
    expect(notAfternoon.includes(janFirst), isTrue);
    expect(notDecemberjson.includes(novLast), isTrue);
    expect(notDecemberjson.includes(decFirst), isFalse);
    expect(notDecemberjson.includes(decLast), isFalse);
    expect(notDecemberjson.includes(janFirst), isTrue);
    expect(notAfternoonjson.includes(novLast), isFalse);
    expect(notAfternoonjson.includes(decFirst), isFalse);
    expect(notAfternoonjson.includes(decLast), isTrue);
    expect(notAfternoonjson.includes(janFirst), isTrue);
  });

  test('TE: Intersect Tests', () {
    DateTime may_first_2012    = new DateTime.utc(2012, 5,  1, 0, 0, 0, 0);
    DateTime june_first_2012   = new DateTime.utc(2012, 6,  1, 0, 0, 0, 0);
    DateTime june_mid_2012     = new DateTime.utc(2012, 6, 15, 10, 11, 12, 13);
    DateTime june_last_2012    = new DateTime.utc(2012, 7, 1, 0, 0, 0, -1);
    DateTime july_last_2012    = new DateTime.utc(2012, 8, 1, 0, 0, 0, -1);

    TEIsAfter start = new TEIsAfter.DateTime(june_first_2012, true);
    TEIsBefore end = new TEIsBefore.DateTime(june_last_2012, false);


    // Everything but June 2012
    TEIntersect intersect = new TEIntersect([start, end]);

    // Test first/last
    expect(intersect.first() == june_first_2012, isTrue);
    expect(intersect.last() == june_last_2012.subtract(new Duration(milliseconds: 1)), isTrue);

    // Test inclusiveness
    expect(intersect.includes(june_mid_2012), isTrue);
    expect(intersect.includes(june_first_2012), isTrue);
    expect(intersect.includes(june_last_2012), isFalse);
    expect(intersect.includes(may_first_2012), isFalse);
    expect(intersect.includes(july_last_2012), isFalse);
  });
}