library testinrange;
import "package:unittest/unittest.dart";
import "package:dartemporal/dartemporal.dart";
import "dart:convert";

testInRange() {
  test('TE: InRange.Year', () {
    DateTime date2012 = new DateTime.utc(2012, 12, 31, 23, 59, 59, 999);   // 2012
    DateTime date2013 = new DateTime.utc(2013, 7, 1, 0, 0, 0, 0);    // 2013
    DateTime date2014 = new DateTime.utc(2014, 1, 1, 0, 0, 0, 0);    // 2014
    DateTime date2015 = new DateTime.utc(2015, 1, 1, 0, 0, 0, 0);    // 2015
    DateTime date2016 = new DateTime.utc(2016, 1, 1, 0, 0, 0, 0);    // 2016

    TEInRange is2013to2015i = new TEInRange.Year(2013,2015);
    TEInRange is2013to2015e = new TEInRange.Year(2013,2015,false,false);
    TEInRange is2013to2015ij = new TEInRange.fromJson(JSON.decode(JSON.encode(is2013to2015i)));
    TEInRange is2013to2015ej = new TEInRange.fromJson(JSON.decode(JSON.encode(is2013to2015e)));

    // Test inclusivity works
    expect(is2013to2015i.includes(date2012), isFalse);
    expect(is2013to2015i.includes(date2013), isTrue);
    expect(is2013to2015i.includes(date2014), isTrue);
    expect(is2013to2015i.includes(date2015), isTrue);
    expect(is2013to2015i.includes(date2016), isFalse);

    expect(is2013to2015ij.includes(date2012), isFalse);
    expect(is2013to2015ij.includes(date2013), isTrue);
    expect(is2013to2015ij.includes(date2014), isTrue);
    expect(is2013to2015ij.includes(date2015), isTrue);
    expect(is2013to2015ij.includes(date2016), isFalse);

    // Test exclusivity works
    expect(is2013to2015e.includes(date2012), isFalse);
    expect(is2013to2015e.includes(date2013), isFalse);
    expect(is2013to2015e.includes(date2014), isTrue);
    expect(is2013to2015e.includes(date2015), isFalse);
    expect(is2013to2015e.includes(date2016), isFalse);

    expect(is2013to2015ej.includes(date2012), isFalse);
    expect(is2013to2015ej.includes(date2013), isFalse);
    expect(is2013to2015ej.includes(date2014), isTrue);
    expect(is2013to2015ej.includes(date2015), isFalse);
    expect(is2013to2015ej.includes(date2016), isFalse);

    // Test first last functions
    expect(is2013to2015i.first() == new DateTime.utc(2013,1,1,0,0,0,0), isTrue);
    expect(is2013to2015i.last()  == new DateTime.utc(2015,12,31,23,59,59,999), isTrue);
    expect(is2013to2015e.first() == new DateTime.utc(2014,1,1,0,0,0,0), isTrue);
    expect(is2013to2015e.last()  == new DateTime.utc(2014,12,31,23,59,59,999), isTrue);

    expect(is2013to2015ij.first() == new DateTime.utc(2013,1,1,0,0,0,0), isTrue);
    expect(is2013to2015ij.last()  == new DateTime.utc(2015,12,31,23,59,59,999), isTrue);
    expect(is2013to2015ej.first() == new DateTime.utc(2014,1,1,0,0,0,0), isTrue);
    expect(is2013to2015ej.last()  == new DateTime.utc(2014,12,31,23,59,59,999), isTrue);
  });

  test('TE: InRange.Month', () {
    DateTime april  = new DateTime.utc(2013, 4, 16, 0, 0, 0, 0);
    DateTime may    = new DateTime.utc(2013, 5,  1, 0, 0, 0, 0);
    DateTime june   = new DateTime.utc(2014, 6, 30, 0, 0, 0, 0);
    DateTime july   = new DateTime.utc(2014, 7, 12, 0, 0, 0, 0);
    DateTime august = new DateTime.utc(2014, 8, 24, 0, 0, 0, 0);

    TEInRange inMaytoJunei = new TEInRange.Month(DateTime.MAY, DateTime.JULY);
    TEInRange inMaytoJunee = new TEInRange.Month(DateTime.MAY, DateTime.JULY, false, false);
    TEInRange inMaytoJuneij = new TEInRange.fromJson(JSON.decode(JSON.encode(inMaytoJunei)));
    TEInRange inMaytoJuneej = new TEInRange.fromJson(JSON.decode(JSON.encode(inMaytoJunee)));

    // Test inclusivity works
    expect(inMaytoJunei.includes(april), isFalse);
    expect(inMaytoJunei.includes(may), isTrue);
    expect(inMaytoJunei.includes(june), isTrue);
    expect(inMaytoJunei.includes(july), isTrue);
    expect(inMaytoJunei.includes(august), isFalse);

    expect(inMaytoJuneij.includes(april), isFalse);
    expect(inMaytoJuneij.includes(may), isTrue);
    expect(inMaytoJuneij.includes(june), isTrue);
    expect(inMaytoJuneij.includes(july), isTrue);
    expect(inMaytoJuneij.includes(august), isFalse);

    // Test exclusivity works
    expect(inMaytoJunee.includes(april), isFalse);
    expect(inMaytoJunee.includes(may), isFalse);
    expect(inMaytoJunee.includes(june), isTrue);
    expect(inMaytoJunee.includes(july), isFalse);
    expect(inMaytoJunee.includes(august), isFalse);

    expect(inMaytoJuneej.includes(april), isFalse);
    expect(inMaytoJuneej.includes(may), isFalse);
    expect(inMaytoJuneej.includes(june), isTrue);
    expect(inMaytoJuneej.includes(july), isFalse);
    expect(inMaytoJuneej.includes(august), isFalse);

    // Test first last functions
    expect(inMaytoJunei.first(), isNull);
    expect(inMaytoJunei.last(), isNull);
    expect(inMaytoJuneij.first(), isNull);
    expect(inMaytoJuneij.last(), isNull);
  });

  test('TE: InRange.DayOfWeek', () {
    DateTime monday    = new DateTime.utc(2013, 6, 17, 0, 0, 0, 0);  // MONDAY
    DateTime tuesday   = new DateTime.utc(2013, 6, 19, 0, 0, 0, 0);   // WEDNESDAY
    DateTime wednesday = new DateTime.utc(2013, 6, 20, 0, 0, 0, 0);   // THURSDAY
    DateTime thursday  = new DateTime.utc(2013, 6, 21, 0, 0, 0, 0);  // FRIDAY
    DateTime friday    = new DateTime.utc(2013, 11, 9, 0, 0, 0, 0);  // SUNDAY

    TEInRange betweenWedAndFrii = new TEInRange.DayOfWeek(DateTime.WEDNESDAY,DateTime.FRIDAY);
    TEInRange betweenWedAndFrie = new TEInRange.DayOfWeek(DateTime.WEDNESDAY,DateTime.FRIDAY,false,false);
    TEInRange betweenWedAndFriij = new TEInRange.fromJson(JSON.decode(JSON.encode(betweenWedAndFrii)));
    TEInRange betweenWedAndFriej = new TEInRange.fromJson(JSON.decode(JSON.encode(betweenWedAndFrie)));

    // Test inclusivity works
    expect(betweenWedAndFrii.includes(monday), isFalse);
    expect(betweenWedAndFrii.includes(tuesday), isTrue);
    expect(betweenWedAndFrii.includes(wednesday), isTrue);
    expect(betweenWedAndFrii.includes(thursday), isTrue);
    expect(betweenWedAndFrii.includes(friday), isFalse);

    expect(betweenWedAndFriij.includes(monday), isFalse);
    expect(betweenWedAndFriij.includes(tuesday), isTrue);
    expect(betweenWedAndFriij.includes(wednesday), isTrue);
    expect(betweenWedAndFriij.includes(thursday), isTrue);
    expect(betweenWedAndFriij.includes(friday), isFalse);

    // Test exclusivity works
    expect(betweenWedAndFrie.includes(monday), isFalse);
    expect(betweenWedAndFrie.includes(tuesday), isFalse);
    expect(betweenWedAndFrie.includes(wednesday), isTrue);
    expect(betweenWedAndFrie.includes(thursday), isFalse);
    expect(betweenWedAndFrie.includes(friday), isFalse);

    expect(betweenWedAndFriej.includes(monday), isFalse);
    expect(betweenWedAndFriej.includes(tuesday), isFalse);
    expect(betweenWedAndFriej.includes(wednesday), isTrue);
    expect(betweenWedAndFriej.includes(thursday), isFalse);
    expect(betweenWedAndFriej.includes(friday), isFalse);

    // Test first last functions
    expect(betweenWedAndFrii.first(), isNull);
    expect(betweenWedAndFrii.last(), isNull);
    expect(betweenWedAndFriij.first(), isNull);
    expect(betweenWedAndFriij.last(), isNull);
  });

  test('TE: InRange.DayOfMonth', () {
    DateTime the9th  = new DateTime.utc(1975, 12, 9, 0, 0, 0, 0);
    DateTime the10th = new DateTime.utc(1975, 1, 10, 0, 0, 0, 0);
    DateTime the13th = new DateTime.utc(1975, 8, 13, 0, 0, 0, 0);
    DateTime the15th = new DateTime.utc(1975, 5, 15, 0, 0, 0, 0);
    DateTime the16th = new DateTime.utc(1975, 10, 16, 0, 0, 0, 0);

    TEInRange between10and15i = new TEInRange.DayOfMonth(10,15);
    TEInRange between10and15e = new TEInRange.DayOfMonth(10,15,false,false);
    TEInRange between10and15ij = new TEInRange.fromJson(JSON.decode(JSON.encode(between10and15i)));
    TEInRange between10and15ej = new TEInRange.fromJson(JSON.decode(JSON.encode(between10and15e)));

    // Test inclusivity works
    expect(between10and15i.includes(the9th), isFalse);
    expect(between10and15i.includes(the10th), isTrue);
    expect(between10and15i.includes(the13th), isTrue);
    expect(between10and15i.includes(the15th), isTrue);
    expect(between10and15i.includes(the16th), isFalse);

    expect(between10and15ij.includes(the9th), isFalse);
    expect(between10and15ij.includes(the10th), isTrue);
    expect(between10and15ij.includes(the13th), isTrue);
    expect(between10and15ij.includes(the15th), isTrue);
    expect(between10and15ij.includes(the16th), isFalse);

    // Test exclusivity works
    expect(between10and15e.includes(the9th), isFalse);
    expect(between10and15e.includes(the10th), isFalse);
    expect(between10and15e.includes(the13th), isTrue);
    expect(between10and15e.includes(the15th), isFalse);
    expect(between10and15e.includes(the16th), isFalse);

    expect(between10and15ej.includes(the9th), isFalse);
    expect(between10and15ej.includes(the10th), isFalse);
    expect(between10and15ej.includes(the13th), isTrue);
    expect(between10and15ej.includes(the15th), isFalse);
    expect(between10and15ej.includes(the16th), isFalse);

    // Test first last functions
    expect(between10and15i.first(), isNull);
    expect(between10and15i.last(), isNull);
    expect(between10and15ij.first(), isNull);
    expect(between10and15ij.last(), isNull);
  });

  test('TE: InRange.DayOfYear', () {
    DateTime day190  = new DateTime.utc(1975, 7,  9, 0, 0, 0, 0);  //190
    DateTime day191  = new DateTime.utc(1975, 7, 10, 0, 0, 0, 0);  //191
    DateTime day192  = new DateTime.utc(1975, 7, 11, 0, 0, 0, 0);  //192
    DateTime day193  = new DateTime.utc(1975, 7, 12, 0, 0, 0, 0);  //193
    DateTime day194  = new DateTime.utc(1975, 7, 13, 0, 0, 0, 0);  //194

    TEInRange days191to193i = new TEInRange.DayOfYear(191,193);
    TEInRange days191to193e = new TEInRange.DayOfYear(191,193,false,false);
    TEInRange days191to193ij = new TEInRange.fromJson(JSON.decode(JSON.encode(days191to193i)));
    TEInRange days191to193ej = new TEInRange.fromJson(JSON.decode(JSON.encode(days191to193e)));

    // Test inclusivity works
    expect(days191to193i.includes(day190), isFalse);
    expect(days191to193i.includes(day191), isTrue);
    expect(days191to193i.includes(day192), isTrue);
    expect(days191to193i.includes(day193), isTrue);
    expect(days191to193i.includes(day194), isFalse);

    expect(days191to193ij.includes(day190), isFalse);
    expect(days191to193ij.includes(day191), isTrue);
    expect(days191to193ij.includes(day192), isTrue);
    expect(days191to193ij.includes(day193), isTrue);
    expect(days191to193ij.includes(day194), isFalse);

    // Test exclusivity works
    expect(days191to193e.includes(day190), isFalse);
    expect(days191to193e.includes(day191), isFalse);
    expect(days191to193e.includes(day192), isTrue);
    expect(days191to193e.includes(day193), isFalse);
    expect(days191to193e.includes(day194), isFalse);

    expect(days191to193ej.includes(day190), isFalse);
    expect(days191to193ej.includes(day191), isFalse);
    expect(days191to193ej.includes(day192), isTrue);
    expect(days191to193ej.includes(day193), isFalse);
    expect(days191to193ej.includes(day194), isFalse);

    // Test first last functions
    expect(days191to193i.first(), isNull);
    expect(days191to193i.last(), isNull);
    expect(days191to193ij.first(), isNull);
    expect(days191to193ij.last(), isNull);
  });

  test('TE: InRange.WeekOfYear', () {
    DateTime week23  = new DateTime.utc(2013, 6,  9, 0, 0, 0, 0);
    DateTime week24  = new DateTime.utc(2013, 6, 16, 0, 0, 0, 0);
    DateTime week25  = new DateTime.utc(2013, 6, 19, 0, 0, 0, 0);
    DateTime week26  = new DateTime.utc(2013, 6, 24, 0, 0, 0, 0);
    DateTime week27  = new DateTime.utc(2013, 7,  1, 0, 0, 0, 0);

    TEInRange inWeeks24to26i = new TEInRange.WeekOfYear(24,26);
    TEInRange inWeeks24to26e = new TEInRange.WeekOfYear(24,26,false,false);

    TEInRange inWeeks24to26ij = new TEInRange.fromJson(JSON.decode(JSON.encode(inWeeks24to26i)));
    TEInRange inWeeks24to26ej = new TEInRange.fromJson(JSON.decode(JSON.encode(inWeeks24to26e)));

    // Test inclusivity works
    expect(inWeeks24to26i.includes(week23), isFalse);
    expect(inWeeks24to26i.includes(week24), isTrue);
    expect(inWeeks24to26i.includes(week25), isTrue);
    expect(inWeeks24to26i.includes(week26), isTrue);
    expect(inWeeks24to26i.includes(week27), isFalse);

    expect(inWeeks24to26ij.includes(week23), isFalse);
    expect(inWeeks24to26ij.includes(week24), isTrue);
    expect(inWeeks24to26ij.includes(week25), isTrue);
    expect(inWeeks24to26ij.includes(week26), isTrue);
    expect(inWeeks24to26ij.includes(week27), isFalse);

    // Test exclusivity works
    expect(inWeeks24to26e.includes(week23), isFalse);
    expect(inWeeks24to26e.includes(week24), isFalse);
    expect(inWeeks24to26e.includes(week25), isTrue);
    expect(inWeeks24to26e.includes(week26), isFalse);
    expect(inWeeks24to26e.includes(week27), isFalse);

    expect(inWeeks24to26ej.includes(week23), isFalse);
    expect(inWeeks24to26ej.includes(week24), isFalse);
    expect(inWeeks24to26ej.includes(week25), isTrue);
    expect(inWeeks24to26ej.includes(week26), isFalse);
    expect(inWeeks24to26ej.includes(week27), isFalse);

    // Test first last functions
    expect(inWeeks24to26i.first(), isNull);
    expect(inWeeks24to26i.last(), isNull);
    expect(inWeeks24to26ij.first(), isNull);
    expect(inWeeks24to26ij.last(), isNull);
  });

  test('TE: InRange.Hour, Minute, Second, Millisecond', () {
    DateTime time1  = new DateTime.utc(2013, 6, 18, 14, 14, 14, 14);
    DateTime time2  = new DateTime.utc(2013, 6, 17, 15, 15, 15, 15);
    DateTime time3  = new DateTime.utc(2013, 6, 16, 16, 16, 16, 16);
    DateTime time4  = new DateTime.utc(2013, 6, 15, 17, 17, 17, 17);
    DateTime time5  = new DateTime.utc(2013, 6, 14, 18, 18, 18, 18);

    TEInRange inH15to17i = new TEInRange.Hour(15,17);
    TEInRange inH15to17e = new TEInRange.Hour(15,17,false,false);
    TEInRange inM15to17i = new TEInRange.Minute(15,17);
    TEInRange inM15to17e = new TEInRange.Minute(15,17,false,false);
    TEInRange inS15to17i = new TEInRange.Second(15,17);
    TEInRange inS15to17e = new TEInRange.Second(15,17,false,false);
    TEInRange inMs15to17i = new TEInRange.Millisecond(15,17);
    TEInRange inMs15to17e = new TEInRange.Millisecond(15,17,false,false);

    TEInRange inH15to17ij = new TEInRange.fromJson(JSON.decode(JSON.encode(inH15to17i)));
    TEInRange inH15to17ej = new TEInRange.fromJson(JSON.decode(JSON.encode(inH15to17e)));
    TEInRange inM15to17ij = new TEInRange.fromJson(JSON.decode(JSON.encode(inM15to17i)));
    TEInRange inM15to17ej = new TEInRange.fromJson(JSON.decode(JSON.encode(inH15to17e)));
    TEInRange inS15to17ij = new TEInRange.fromJson(JSON.decode(JSON.encode(inS15to17i)));
    TEInRange inS15to17ej = new TEInRange.fromJson(JSON.decode(JSON.encode(inS15to17e)));
    TEInRange inMs15to17ij = new TEInRange.fromJson(JSON.decode(JSON.encode(inMs15to17i)));
    TEInRange inMs15to17ej = new TEInRange.fromJson(JSON.decode(JSON.encode(inMs15to17e)));;


    // Test inclusivity works
    expect(inH15to17i.includes(time1), isFalse);
    expect(inH15to17i.includes(time2), isTrue);
    expect(inH15to17i.includes(time3), isTrue);
    expect(inH15to17i.includes(time4), isTrue);
    expect(inH15to17i.includes(time5), isFalse);

    expect(inM15to17i.includes(time1), isFalse);
    expect(inM15to17i.includes(time2), isTrue);
    expect(inM15to17i.includes(time3), isTrue);
    expect(inM15to17i.includes(time4), isTrue);
    expect(inM15to17i.includes(time5), isFalse);

    expect(inS15to17i.includes(time1), isFalse);
    expect(inS15to17i.includes(time2), isTrue);
    expect(inS15to17i.includes(time3), isTrue);
    expect(inS15to17i.includes(time4), isTrue);
    expect(inS15to17i.includes(time5), isFalse);

    expect(inMs15to17i.includes(time1), isFalse);
    expect(inMs15to17i.includes(time2), isTrue);
    expect(inMs15to17i.includes(time3), isTrue);
    expect(inMs15to17i.includes(time4), isTrue);
    expect(inMs15to17i.includes(time5), isFalse);

    expect(inH15to17ij.includes(time1), isFalse);
    expect(inH15to17ij.includes(time2), isTrue);
    expect(inH15to17ij.includes(time3), isTrue);
    expect(inH15to17ij.includes(time4), isTrue);
    expect(inH15to17ij.includes(time5), isFalse);

    expect(inM15to17ij.includes(time1), isFalse);
    expect(inM15to17ij.includes(time2), isTrue);
    expect(inM15to17ij.includes(time3), isTrue);
    expect(inM15to17ij.includes(time4), isTrue);
    expect(inM15to17ij.includes(time5), isFalse);

    expect(inS15to17ij.includes(time1), isFalse);
    expect(inS15to17ij.includes(time2), isTrue);
    expect(inS15to17ij.includes(time3), isTrue);
    expect(inS15to17ij.includes(time4), isTrue);
    expect(inS15to17ij.includes(time5), isFalse);

    expect(inMs15to17ij.includes(time1), isFalse);
    expect(inMs15to17ij.includes(time2), isTrue);
    expect(inMs15to17ij.includes(time3), isTrue);
    expect(inMs15to17ij.includes(time4), isTrue);
    expect(inMs15to17ij.includes(time5), isFalse);

    // Test exclusivity works
    expect(inH15to17e.includes(time1), isFalse);
    expect(inH15to17e.includes(time2), isFalse);
    expect(inH15to17e.includes(time3), isTrue);
    expect(inH15to17e.includes(time4), isFalse);
    expect(inH15to17e.includes(time5), isFalse);

    expect(inM15to17e.includes(time1), isFalse);
    expect(inM15to17e.includes(time2), isFalse);
    expect(inM15to17e.includes(time3), isTrue);
    expect(inM15to17e.includes(time4), isFalse);
    expect(inM15to17e.includes(time5), isFalse);

    expect(inS15to17e.includes(time1), isFalse);
    expect(inS15to17e.includes(time2), isFalse);
    expect(inS15to17e.includes(time3), isTrue);
    expect(inS15to17e.includes(time4), isFalse);
    expect(inS15to17e.includes(time5), isFalse);

    expect(inMs15to17e.includes(time1), isFalse);
    expect(inMs15to17e.includes(time2), isFalse);
    expect(inMs15to17e.includes(time3), isTrue);
    expect(inMs15to17e.includes(time4), isFalse);
    expect(inMs15to17e.includes(time5), isFalse);

    expect(inH15to17ej.includes(time1), isFalse);
    expect(inH15to17ej.includes(time2), isFalse);
    expect(inH15to17ej.includes(time3), isTrue);
    expect(inH15to17ej.includes(time4), isFalse);
    expect(inH15to17ej.includes(time5), isFalse);

    expect(inM15to17ej.includes(time1), isFalse);
    expect(inM15to17ej.includes(time2), isFalse);
    expect(inM15to17ej.includes(time3), isTrue);
    expect(inM15to17ej.includes(time4), isFalse);
    expect(inM15to17ej.includes(time5), isFalse);

    expect(inS15to17ej.includes(time1), isFalse);
    expect(inS15to17ej.includes(time2), isFalse);
    expect(inS15to17ej.includes(time3), isTrue);
    expect(inS15to17ej.includes(time4), isFalse);
    expect(inS15to17ej.includes(time5), isFalse);

    expect(inMs15to17ej.includes(time1), isFalse);
    expect(inMs15to17ej.includes(time2), isFalse);
    expect(inMs15to17ej.includes(time3), isTrue);
    expect(inMs15to17ej.includes(time4), isFalse);
    expect(inMs15to17ej.includes(time5), isFalse);


    // Test first last functions
    expect(inH15to17i.first(), isNull);
    expect(inH15to17i.last(), isNull);
    expect(inM15to17e.first(), isNull);
    expect(inM15to17e.last(), isNull);
    expect(inS15to17i.first(), isNull);
    expect(inS15to17i.last(), isNull);
    expect(inMs15to17e.first(), isNull);
    expect(inMs15to17e.last(), isNull);

    expect(inH15to17ij.first(), isNull);
    expect(inH15to17ij.last(), isNull);
    expect(inM15to17ej.first(), isNull);
    expect(inM15to17ej.last(), isNull);
    expect(inS15to17ij.first(), isNull);
    expect(inS15to17ij.last(), isNull);
    expect(inMs15to17ej.first(), isNull);
    expect(inMs15to17ej.last(), isNull);
  });

  test('TE: InRange.DateTime', () {
    DateTime before = new DateTime.utc(2013, 7, 11, 0, 0, 0, -1);
    DateTime edgeb = new DateTime.utc(2013, 7, 11, 0, 0, 0, 0);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 1);
    DateTime edgea  = new DateTime.utc(2013, 7, 11, 0, 0, 0, 2);
    DateTime after  = new DateTime.utc(2013, 7, 11, 0, 0, 0, 3);

    TEInRange targetDatei = new TEInRange.DateTime(edgeb,edgea);
    TEInRange targetDatee = new TEInRange.DateTime(edgeb,edgea,false,false);

    TEInRange targetDateij = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatei)));
    TEInRange targetDateej = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatee)));

    // Test inclusivity works
    expect(targetDatei.includes(before), isFalse);
    expect(targetDatei.includes(edgeb), isTrue);
    expect(targetDatei.includes(target), isTrue);
    expect(targetDatei.includes(edgea), isTrue);
    expect(targetDatei.includes(after), isFalse);

    expect(targetDateij.includes(before), isFalse);
    expect(targetDateij.includes(edgeb), isTrue);
    expect(targetDateij.includes(target), isTrue);
    expect(targetDateij.includes(edgea), isTrue);
    expect(targetDateij.includes(after), isFalse);


    // Test exclusivity works
    expect(targetDatee.includes(before), isFalse);
    expect(targetDatee.includes(edgeb), isFalse);
    expect(targetDatee.includes(target), isTrue);
    expect(targetDatee.includes(edgea), isFalse);
    expect(targetDatee.includes(after), isFalse);

    expect(targetDateej.includes(before), isFalse);
    expect(targetDateej.includes(edgeb), isFalse);
    expect(targetDateej.includes(target), isTrue);
    expect(targetDateej.includes(edgea), isFalse);
    expect(targetDateej.includes(after), isFalse);

    // Test first last functions
    expect(targetDatei.first(), edgeb);
    expect(targetDatei.last(), edgea);
    expect(targetDatee.first(), edgeb.add(new Duration(milliseconds:1)));
    expect(targetDatee.last(), edgea.add(new Duration(milliseconds:-1)));

    expect(targetDateij.first(), edgeb);
    expect(targetDateij.last(), edgea);
    expect(targetDateej.first(), edgeb.add(new Duration(milliseconds:1)));
    expect(targetDateej.last(), edgea.add(new Duration(milliseconds:-1)));
  });

  test('TE: InRange.Date', () {
    DateTime before = new DateTime.utc(2013, 7, 9, 0, 0, 0, 0);
    DateTime edgeb = new DateTime.utc(2013, 7, 10, 0, 0, 0, 0);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 0);
    DateTime edgea  = new DateTime.utc(2013, 7, 12, 0, 0, 0, 0);
    DateTime after  = new DateTime.utc(2013, 7, 13, 0, 0, 0, 0);

    TEInRange targetDatei = new TEInRange.DateTime(edgeb,edgea);
    TEInRange targetDatee = new TEInRange.DateTime(edgeb,edgea,false,false);
    TEInRange targetDateij = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatei)));
    TEInRange targetDateej = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatee)));

    // Test inclusivity works
    expect(targetDatei.includes(before), isFalse);
    expect(targetDatei.includes(edgeb), isTrue);
    expect(targetDatei.includes(target), isTrue);
    expect(targetDatei.includes(edgea), isTrue);
    expect(targetDatei.includes(after), isFalse);

    expect(targetDateij.includes(before), isFalse);
    expect(targetDateij.includes(edgeb), isTrue);
    expect(targetDateij.includes(target), isTrue);
    expect(targetDateij.includes(edgea), isTrue);
    expect(targetDateij.includes(after), isFalse);


    // Test exclusivity works
    expect(targetDatee.includes(before), isFalse);
    expect(targetDatee.includes(edgeb), isFalse);
    expect(targetDatee.includes(target), isTrue);
    expect(targetDatee.includes(edgea), isFalse);
    expect(targetDatee.includes(after), isFalse);

    expect(targetDateej.includes(before), isFalse);
    expect(targetDateej.includes(edgeb), isFalse);
    expect(targetDateej.includes(target), isTrue);
    expect(targetDateej.includes(edgea), isFalse);
    expect(targetDateej.includes(after), isFalse);

    // Test first last functions
    expect(targetDatei.first(), edgeb);
    expect(targetDatei.last(), edgea);
    expect(targetDatee.first(), edgeb.add(new Duration(milliseconds:1)));
    expect(targetDatee.last(), edgea.add(new Duration(milliseconds:-1)));

    expect(targetDateij.first(), edgeb);
    expect(targetDateij.last(), edgea);
    expect(targetDateej.first(), edgeb.add(new Duration(milliseconds:1)));
    expect(targetDateej.last(), edgea.add(new Duration(milliseconds:-1)));
  });
  
  test('TE: InRange.openings', () {
    DateTime before = new DateTime.utc(2013, 7, 9, 0, 0, 0, 0);
    DateTime edgeb = new DateTime.utc(2013, 7, 10, 0, 0, 0, 0);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 0);
    DateTime edgea  = new DateTime.utc(2013, 7, 12, 0, 0, 0, 0);
    DateTime after  = new DateTime.utc(2013, 7, 13, 0, 0, 0, 0);

    TEInRange targetDatei = new TEInRange.DateTime(edgeb,edgea);
    TEInRange targetDatee = new TEInRange.DateTime(edgeb,edgea,false,false);
    TEInRange targetDateij = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatei)));
    TEInRange targetDateej = new TEInRange.fromJson(JSON.decode(JSON.encode(targetDatee)));
    
    targetDatei.openings(edgeb, new Duration(hours:2), new Duration(minutes:1), new Duration(minutes:5), new Duration(minutes:30)).then((result) {
      expect(result[0].first(), new DateTime.utc(2013, 7, 10, 0, 0, 0, 0));
      expect(result[9].last(), new DateTime.utc(2013, 7, 10, 6, 30, 0, 0));
    });
    
    targetDatee.openings(edgeb, new Duration(hours:2), new Duration(minutes:1), new Duration(minutes:5), new Duration(minutes:30), 5).then((result) {
      expect(result[0].first(), new DateTime.utc(2013, 7, 10, 0, 5, 0, 0));
      expect(result[4].last(), new DateTime.utc(2013, 7, 10, 4, 5, 0, 0));
    });;
  });
}