library testis;
import "package:unittest/unittest.dart";
import "package:dartemporal/dartemporal.dart";
import "dart:convert";

testIs() {
  group('TEIs -> ', () {

    // This group of tests checks just the type() methods of each of the TE instantiators
    //   The values in the constructors do not really matter, they are just to get an object
    //   to test against.
    test('type() methods', () {
      expect(new TEIs.Year(2012).type(), TExpression.YEAR);
      expect(new TEIs.Month(3).type(), TExpression.MONTH);
      expect(new TEIs.DateTime(new DateTime.now()).type(), TExpression.DATETIME);
      expect(new TEIs.WeekOfYear(35).type(), TExpression.WEEKOFYEAR);
      expect(new TEIs.DayOfWeek(3).type(), TExpression.DAYOFWEEK);
      expect(new TEIs.DayOfMonth(22).type(), TExpression.DAYOFMONTH);
      expect(new TEIs.DayOfYear(123).type(), TExpression.DAYOFYEAR);
      expect(new TEIs.Hour(22).type(), TExpression.HOUR);
      expect(new TEIs.Minute(33).type(), TExpression.MINUTE);
      expect(new TEIs.Second(45).type(), TExpression.SECOND);
      expect(new TEIs.Millisecond(333).type(), TExpression.MILLISECOND);
      expect(new TEIs.Date(new DateTime.now()).type(), TExpression.DATE);
      expect(new TEIs.WeekOfMonth(-2).type(), TExpression.WEEKOFMONTH);
    });

    // Tests for the TEIs.Year() constructor
    test('Year() constructor', () {
      DateTime testDate;

      // We will test times every 23 1/3 hours one thousand times
      //  This lets us hit a whole bunch of dates/times
      [-2000, 0, 101, 2012, 5313].forEach((y){
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Year(y);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);
        DateTime yearStart = new DateTime.utc(y,1,1,0,0,0,0);
        DateTime yearEnd = new DateTime.utc(y+1,1,1,0,0,0,-1);

        // Test first last functions
        expect(testTE.first(), yearStart);
        expect(testTE.last(), yearEnd);
        expect(testTEjson.first(), yearStart);
        expect(testTEjson.last(), yearEnd);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.year == y);
          expect(testTEjson.includes(testDate), testDate.year == y);
        }
      });
    });

  // Test for the TE: Is.Month() constructor
  test('Month() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of various years
    [-2000, 0, 101, 2012, 5313].forEach((y){
      // test for each month of the year
      for(int m=1; m<13; m++) {
        /// We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Month(m);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), null);
        expect(testTE.last(), null);
        expect(testTEjson.first(), null);
        expect(testTEjson.last(), null);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.month == m);
          expect(testTEjson.includes(testDate), testDate.month == m);
        }
      }
    });
  });

  // Test for the TE: Is.DayOfWeek() constructor
  test('DayOfWeek() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dow=0; dow<7; dow++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.DayOfWeek(dow);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.weekday == dow);
          expect(testTEjson.includes(testDate), testDate.weekday == dow);
        }
      }
    });
  });

  // Test for the TE: Is.DayOfMonth() constructor
  test('DayOfMonth() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dom=1; dom<32; dom++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.DayOfMonth(dom);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.day == dom);
          expect(testTEjson.includes(testDate), testDate.day == dom);
        }
      }
    });
  });

  // Test for the TE: Is.DayOfYear() constructor
  test('DayOfYear() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int doy=1; doy<366; doy++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.DayOfYear(doy);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y,1,1,0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        int daycounter = 1;

        // Test inclusivity works
        for(int i=0; i < 365; i++) {
          testDate = base.add(new Duration(hours: 24 * i));
          expect(testTE.includes(testDate), daycounter == doy);
          expect(testTEjson.includes(testDate), daycounter == doy);
          daycounter += 1;
        }
      }
    });
  });

  // Test for the TE: Is.WeekOfYear() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfYear() constructor', () {
    DateTime week25  = new DateTime(2013, 6, 19, 0, 0, 0, 0);
    DateTime week24  = new DateTime(2013, 6, 16, 0, 0, 0, 0);
    DateTime week26  = new DateTime(2013, 6, 24, 0, 0, 0, 0);

    TEIs onWeek25 = new TEIs.WeekOfYear(25);
    TEIs onWeek25json = new TEIs.fromJson(JSON.decode(JSON.encode(onWeek25)));

    // Test inclusivity works
    expect(onWeek25.includes(week24), isFalse);
    expect(onWeek25.includes(week25), isTrue);
    expect(onWeek25.includes(week26), isFalse);
    expect(onWeek25json.includes(week24), isFalse);
    expect(onWeek25json.includes(week25), isTrue);
    expect(onWeek25json.includes(week26), isFalse);

    // Test first last functions
    expect(onWeek25.first(), isNull);
    expect(onWeek25.last(), isNull);
    expect(onWeek25json.first(), isNull);
    expect(onWeek25json.last(), isNull);
  });

  // Tests for the TEIs.Hour() constructor
  test('Hour() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int h=0; h<24; h++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Hour(h);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.hour == h);
          expect(testTEjson.includes(testDate), testDate.hour == h);
        }
      }
    });
  });

  // Tests for the TEIs.Minute() constructor
  test('Minute() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int m=0; m<60; m++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Minute(m);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.minute == m);
          expect(testTEjson.includes(testDate), testDate.minute == m);
        }
      }
    });
  });

  // Tests for the TEIs.Second() constructor
  test('Second() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int s=0; s<60; s++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Second(s);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.second == s);
          expect(testTEjson.includes(testDate), testDate.second == s);
        }
      }
    });
  });

  // Tests for the TEIs.Millisecond() constructor
  test('Millisecond() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIs testTE = new TEIs.Millisecond(ms);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1007 * i));
          expect(testTE.includes(testDate), testDate.millisecond == ms);
          expect(testTEjson.includes(testDate), testDate.millisecond == ms);
        }
      }
    });
  });

  // Tests for the TEIs.DateTime() constructor
  test('DateTime() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIs testTE = new TEIs.DateTime(base);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));

        // Test first last functions
        expect(testTE.first(), base);
        expect(testTE.last(), base);
        expect(testTEjson.first(), base);
        expect(testTEjson.last(), base);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1 * i));
          expect(testTE.includes(testDate), testDate == base);
          expect(testTEjson.includes(testDate), testDate == base);
        }
      }
    });
  });

  // Tests for the TEIs.Date() constructor
  test('Date() constructor', () {
    DateTime testDate;
    DateTime dayStart;
    DateTime dayEnd;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIs testTE = new TEIs.Date(base);
        TEIs testTEjson = new TEIs.fromJson(JSON.decode(JSON.encode(testTE)));
        dayStart = new DateTime.utc(base.year, base.month, base.day, 0,0,0,0);
        dayEnd   = new DateTime.utc(base.year, base.month, base.day, 23,59,59,999);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(hours:95 * i));

          expect(testTE.includes(testDate), testDate == base);
          expect(testTEjson.includes(testDate), testDate == base);
          expect(testTE.first(), dayStart);
          expect(testTEjson.first(), dayStart);
          expect(testTE.last(), dayEnd);
          expect(testTEjson.last(), dayEnd);
        }
      }
    });
  });

  // Test for the TE: Is.WeekOfMonth() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfMonth() constructor', () {
    DateTime week1 = new DateTime.utc(2013, 7,  3, 0, 0, 0, 0);  // Week 1 of 5
    DateTime week2 = new DateTime.utc(2013, 7, 10, 0, 0, 0, 0); // Week 2 of 5
    DateTime week3  = new DateTime.utc(2013, 7, 17, 0, 0, 0, 0); // Week 3 of 5
    DateTime week4 = new DateTime.utc(2013, 7, 24, 0, 0, 0, 0); // Week 4 of 5
    DateTime week5  = new DateTime.utc(2013, 7, 31, 0, 0, 0, 0); // Week 5 of 5

    TEIs targetWeek0 = new TEIs.WeekOfMonth(0);
    TEIs targetWeek1 = new TEIs.WeekOfMonth(3);
    TEIs targetWeek2 = new TEIs.WeekOfMonth(-2);
    TEIs targetWeek0json = new TEIs.fromJson(JSON.decode(JSON.encode(targetWeek0)));
    TEIs targetWeek1json = new TEIs.fromJson(JSON.decode(JSON.encode(targetWeek1)));
    TEIs targetWeek2json = new TEIs.fromJson(JSON.decode(JSON.encode(targetWeek2)));

    // Test inclusivity works
    expect(targetWeek0.includes(week1), isFalse);
    expect(targetWeek0.includes(week2), isFalse);
    expect(targetWeek0.includes(week3), isFalse);
    expect(targetWeek0.includes(week4), isFalse);
    expect(targetWeek0.includes(week5), isFalse);

    expect(targetWeek1.includes(week1), isFalse);
    expect(targetWeek1.includes(week2), isFalse);
    expect(targetWeek1.includes(week3), isTrue);
    expect(targetWeek1.includes(week4), isFalse);
    expect(targetWeek1.includes(week5), isFalse);

    expect(targetWeek2.includes(week1), isFalse);
    expect(targetWeek2.includes(week2), isFalse);
    expect(targetWeek2.includes(week3), isFalse);
    expect(targetWeek2.includes(week4), isTrue);
    expect(targetWeek2.includes(week5), isFalse);

    expect(targetWeek0json.includes(week1), isFalse);
    expect(targetWeek0json.includes(week2), isFalse);
    expect(targetWeek0json.includes(week3), isFalse);
    expect(targetWeek0json.includes(week4), isFalse);
    expect(targetWeek0json.includes(week5), isFalse);

    expect(targetWeek1json.includes(week1), isFalse);
    expect(targetWeek1json.includes(week2), isFalse);
    expect(targetWeek1json.includes(week3), isTrue);
    expect(targetWeek1json.includes(week4), isFalse);
    expect(targetWeek1json.includes(week5), isFalse);

    expect(targetWeek2json.includes(week1), isFalse);
    expect(targetWeek2json.includes(week2), isFalse);
    expect(targetWeek2json.includes(week3), isFalse);
    expect(targetWeek2json.includes(week4), isTrue);
    expect(targetWeek2json.includes(week5), isFalse);

    // Test first last functions
    expect(targetWeek1.first(), isNull);
    expect(targetWeek1.last(), isNull);
    expect(targetWeek1json.first(), isNull);
    expect(targetWeek1json.last(), isNull);
  });


  // Test for the TE: Is.dates() method
  //   TODO: This test group could use some more meat.
  test('dates() method', () {
    DateTime base   = new DateTime.utc(2013, 7, 11, 0, 0, 0, -30);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 1);

    TEIs targetDate = new TEIs.DateTime(target);
    TEIs targetDatejson = new TEIs.fromJson(JSON.decode(JSON.encode(targetDate)));
    TEIs targetMonth = new TEIs.Month(8);
    TEIs targetMonthjson = new TEIs.fromJson(JSON.decode(JSON.encode(targetMonth)));

    List<DateTime> results;
    targetDate.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results, [target]);
    });

    targetDatejson.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results, [target]);
    });

    // Test the result count limiting parameter
    targetMonth.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });

    // Test the iteration limiting parameter
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
  });
  });
}