library testisafter;
import "package:unittest/unittest.dart";
import "package:dartemporal/dartemporal.dart";
import "dart:convert";

testIsAfter() {
  group('TEIsAfter -> ', () {

    // This group of tests checks just the type() methods of each of the TE instantiators
    //   The values in the constructors do not really matter, they are just to get an object
    //   to test against.
    test('type() methods', () {
      expect(new TEIsAfter.Year(2012).type(), TExpression.YEAR);
      expect(new TEIsAfter.Month(3).type(), TExpression.MONTH);
      expect(new TEIsAfter.DateTime(new DateTime.now()).type(), TExpression.DATETIME);
      expect(new TEIsAfter.WeekOfYear(35).type(), TExpression.WEEKOFYEAR);
      expect(new TEIsAfter.DayOfWeek(3).type(), TExpression.DAYOFWEEK);
      expect(new TEIsAfter.DayOfMonth(22).type(), TExpression.DAYOFMONTH);
      expect(new TEIsAfter.DayOfYear(123).type(), TExpression.DAYOFYEAR);
      expect(new TEIsAfter.Hour(22).type(), TExpression.HOUR);
      expect(new TEIsAfter.Minute(33).type(), TExpression.MINUTE);
      expect(new TEIsAfter.Second(45).type(), TExpression.SECOND);
      expect(new TEIsAfter.Millisecond(333).type(), TExpression.MILLISECOND);
      expect(new TEIsAfter.Date(new DateTime.now()).type(), TExpression.DATE);
      expect(new TEIsAfter.WeekOfMonth(-2).type(), TExpression.WEEKOFMONTH);
    });

    // Tests for the TEIsAfter.Year() constructor
    test('Year() constructor', () {
      DateTime testDate;

      // We will test times every 23 1/3 hours one thousand times
      //  This lets us hit a whole bunch of dates/times
      [-2000, 0, 101, 2012, 5313].forEach((y){
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Year(y);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Year(y, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);
        DateTime yearStart = new DateTime.utc(y,1,1,0,0,0,0);
        DateTime yearStartni = new DateTime.utc(y+1,1,1,0,0,0,0);

        // Test first last functions
        expect(testTE.first(), yearStart);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), yearStartni);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.year >= y);
          expect(testTEjson.includes(testDate), testDate.year >= y);
          expect(testTEni.includes(testDate), testDate.year > y);
          expect(testTEnijson.includes(testDate), testDate.year > y);
        }
      });
    });

  // Test for the TE: TEIsAfter.Month() constructor
  test('Month() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of various years
    [-2000, 0, 101, 2012, 5313].forEach((y){
      // test for each month of the year
      for(int m=1; m<13; m++) {
        /// We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Month(m);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Month(m, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.month >= m);
          expect(testTEjson.includes(testDate), testDate.month >= m);
          expect(testTEni.includes(testDate), testDate.month > m);
          expect(testTEnijson.includes(testDate), testDate.month > m);
        }
      }
    });
  });

  // Test for the TE: Is.DayOfWeek() constructor
  test('DayOfWeek() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dow=0; dow<7; dow++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.DayOfWeek(dow);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.DayOfWeek(dow, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.weekday >= dow);
          expect(testTEjson.includes(testDate), testDate.weekday >= dow);
          expect(testTEni.includes(testDate), testDate.weekday > dow);
          expect(testTEnijson.includes(testDate), testDate.weekday > dow);
        }
      }
    });
  });

  // Test for the TE: TEIsAfter.DayOfMonth() constructor
  test('DayOfMonth() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dom=1; dom<32; dom++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.DayOfMonth(dom);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.DayOfMonth(dom, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.day >= dom);
          expect(testTEjson.includes(testDate), testDate.day >= dom);
          expect(testTEni.includes(testDate), testDate.day > dom);
          expect(testTEnijson.includes(testDate), testDate.day > dom);
        }
      }
    });
  });

  // Test for the TE: TEIsAfter.DayOfYear() constructor
  test('DayOfYear() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int doy=1; doy<366; doy++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.DayOfYear(doy);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.DayOfYear(doy, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y,1,1,0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        int daycounter = 1;

        // Test inclusivity works
        for(int i=0; i < 365; i++) {
          testDate = base.add(new Duration(hours: 24 * i));
          expect(testTE.includes(testDate), daycounter >= doy);
          expect(testTEjson.includes(testDate), daycounter >= doy);
          expect(testTEni.includes(testDate), daycounter > doy);
          expect(testTEnijson.includes(testDate), daycounter > doy);
          daycounter += 1;
        }
      }
    });
  });

  // Test for the TE: TEIsAfter.WeekOfYear() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfYear() constructor', () {
    DateTime week25  = new DateTime(2013, 6, 19, 0, 0, 0, 0);
    DateTime week24  = new DateTime(2013, 6, 16, 0, 0, 0, 0);
    DateTime week26  = new DateTime(2013, 6, 24, 0, 0, 0, 0);

    TEIsAfter onWeek25 = new TEIsAfter.WeekOfYear(25);
    TEIsAfter onWeek25json = new TEIsAfter.fromJson(JSON.decode(JSON.encode(onWeek25)));
    TEIsAfter onWeek25ni = new TEIsAfter.WeekOfYear(25,false);
    TEIsAfter onWeek25nijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(onWeek25ni)));

    // Test inclusivity works
    expect(onWeek25.includes(week24), isFalse);
    expect(onWeek25.includes(week25), isTrue);
    expect(onWeek25.includes(week26), isTrue);
    expect(onWeek25json.includes(week24), isFalse);
    expect(onWeek25json.includes(week25), isTrue);
    expect(onWeek25json.includes(week26), isTrue);
    expect(onWeek25ni.includes(week24), isFalse);
    expect(onWeek25ni.includes(week25), isFalse);
    expect(onWeek25ni.includes(week26), isTrue);
    expect(onWeek25nijson.includes(week24), isFalse);
    expect(onWeek25nijson.includes(week25), isFalse);
    expect(onWeek25nijson.includes(week26), isTrue);

    // Test first last functions
    expect(onWeek25.first(), isNull);
    expect(onWeek25.last(), isNull);
    expect(onWeek25json.first(), isNull);
    expect(onWeek25json.last(), isNull);
    expect(onWeek25ni.first(), isNull);
    expect(onWeek25ni.last(), isNull);
    expect(onWeek25nijson.first(), isNull);
    expect(onWeek25nijson.last(), isNull);
  });

  // Tests for the TEIsAfter.Hour() constructor
  test('Hour() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int h=0; h<24; h++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Hour(h);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Hour(h, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.hour >= h);
          expect(testTEjson.includes(testDate), testDate.hour >= h);
          expect(testTEni.includes(testDate), testDate.hour > h);
          expect(testTEnijson.includes(testDate), testDate.hour > h);
        }
      }
    });
  });

  // Tests for the TEIsAfter.Minute() constructor
  test('Minute() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int m=0; m<60; m++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Minute(m);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Minute(m, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.minute >= m);
          expect(testTEjson.includes(testDate), testDate.minute >= m);
          expect(testTEni.includes(testDate), testDate.minute > m);
          expect(testTEnijson.includes(testDate), testDate.minute > m);
        }
      }
    });
  });

  // Tests for the TEIsAfter.Second() constructor
  test('Second() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int s=0; s<60; s++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Second(s);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Second(s, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.second >= s);
          expect(testTEjson.includes(testDate), testDate.second >= s);
          expect(testTEni.includes(testDate), testDate.second > s);
          expect(testTEnijson.includes(testDate), testDate.second > s);
        }
      }
    });
  });

  // Tests for the TEIsAfter.Millisecond() constructor
  test('Millisecond() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsAfter testTE = new TEIsAfter.Millisecond(ms);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Millisecond(ms, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1007 * i));
          expect(testTE.includes(testDate), testDate.millisecond >= ms);
          expect(testTEjson.includes(testDate), testDate.millisecond >= ms);
          expect(testTEni.includes(testDate), testDate.millisecond > ms);
          expect(testTEnijson.includes(testDate), testDate.millisecond > ms);
        }
      }
    });
  });

  // Tests for the TEIsAfter.DateTime() constructor
  test('DateTime() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIsAfter testTE = new TEIsAfter.DateTime(base);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.DateTime(base, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        // Test first last functions
        expect(testTE.first(), base);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), base);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), base.add(new Duration(milliseconds: 1)));
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), base.add(new Duration(milliseconds: 1)));
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1 * i));
          expect(testTE.includes(testDate), testDate.millisecondsSinceEpoch >= base.millisecondsSinceEpoch);
          expect(testTEjson.includes(testDate), testDate.millisecondsSinceEpoch >= base.millisecondsSinceEpoch);
          expect(testTEni.includes(testDate), testDate.millisecondsSinceEpoch > base.millisecondsSinceEpoch);
          expect(testTEnijson.includes(testDate), testDate.millisecondsSinceEpoch > base.millisecondsSinceEpoch);
        }
      }
    });
  });

  // Tests for the TEIsAfter.Date() constructor
  test('Date() constructor', () {
    DateTime testDate;
    DateTime dayStart;
    DateTime dayEnd;
    DateTime dayStartni;
    DateTime dayEndni;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIsAfter testTE = new TEIsAfter.Date(base);
        TEIsAfter testTEjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsAfter testTEni = new TEIsAfter.Date(base, false);
        TEIsAfter testTEnijson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(testTEni)));

        dayStart = new DateTime.utc(base.year, base.month, base.day, 0,0,0,0);
        dayStartni = new DateTime.utc(base.year, base.month, base.day+1, 0,0,0,0);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(hours:95 * i));

          expect(testTE.includes(testDate), testDate.millisecondsSinceEpoch >= base.millisecondsSinceEpoch);
          expect(testTEjson.includes(testDate), testDate.millisecondsSinceEpoch >= base.millisecondsSinceEpoch);

          expect(testTE.first(), dayStart);
          expect(testTEjson.first(), dayStart);
          expect(testTE.last(), isNull);
          expect(testTEjson.last(), isNull);

          expect(testTEni.includes(testDate), testDate.millisecondsSinceEpoch > base.millisecondsSinceEpoch);
          expect(testTEnijson.includes(testDate), testDate.millisecondsSinceEpoch > base.millisecondsSinceEpoch);

          expect(testTEni.first(), dayStartni);
          expect(testTEnijson.first(), dayStartni);
          expect(testTEni.last(), isNull);
          expect(testTEnijson.last(), isNull);
        }
      }
    });
  });

  // Test for the TE: TEIsAfter.WeekOfMonth() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfMonth() constructor', () {
    DateTime week1 = new DateTime.utc(2013, 7,  3, 0, 0, 0, 0);  // Week 1 of 5
    DateTime week2 = new DateTime.utc(2013, 7, 10, 0, 0, 0, 0); // Week 2 of 5
    DateTime week3  = new DateTime.utc(2013, 7, 17, 0, 0, 0, 0); // Week 3 of 5
    DateTime week4 = new DateTime.utc(2013, 7, 24, 0, 0, 0, 0); // Week 4 of 5
    DateTime week5  = new DateTime.utc(2013, 7, 31, 0, 0, 0, 0); // Week 5 of 5

    TEIsAfter targetWeek0 = new TEIsAfter.WeekOfMonth(0);
    TEIsAfter targetWeek1 = new TEIsAfter.WeekOfMonth(3);
    TEIsAfter targetWeek2 = new TEIsAfter.WeekOfMonth(-2);
    TEIsAfter targetWeek0json = new TEIsAfter.fromJson(JSON.decode(JSON.encode(targetWeek0)));
    TEIsAfter targetWeek1json = new TEIsAfter.fromJson(JSON.decode(JSON.encode(targetWeek1)));
    TEIsAfter targetWeek2json = new TEIsAfter.fromJson(JSON.decode(JSON.encode(targetWeek2)));

    // Test inclusivity works
    expect(targetWeek0.includes(week1), isFalse);
    expect(targetWeek0.includes(week2), isFalse);
    expect(targetWeek0.includes(week3), isFalse);
    expect(targetWeek0.includes(week4), isFalse);
    expect(targetWeek0.includes(week5), isFalse);

    expect(targetWeek1.includes(week1), isFalse);
    expect(targetWeek1.includes(week2), isFalse);
    expect(targetWeek1.includes(week3), isTrue);
    expect(targetWeek1.includes(week4), isTrue);
    expect(targetWeek1.includes(week5), isTrue);

    expect(targetWeek2.includes(week1), isFalse);
    expect(targetWeek2.includes(week2), isFalse);
    expect(targetWeek2.includes(week3), isFalse);
    expect(targetWeek2.includes(week4), isTrue);
    expect(targetWeek2.includes(week5), isTrue);

    expect(targetWeek0json.includes(week1), isFalse);
    expect(targetWeek0json.includes(week2), isFalse);
    expect(targetWeek0json.includes(week3), isFalse);
    expect(targetWeek0json.includes(week4), isFalse);
    expect(targetWeek0json.includes(week5), isFalse);

    expect(targetWeek1json.includes(week1), isFalse);
    expect(targetWeek1json.includes(week2), isFalse);
    expect(targetWeek1json.includes(week3), isTrue);
    expect(targetWeek1json.includes(week4), isTrue);
    expect(targetWeek1json.includes(week5), isTrue);

    expect(targetWeek2json.includes(week1), isFalse);
    expect(targetWeek2json.includes(week2), isFalse);
    expect(targetWeek2json.includes(week3), isFalse);
    expect(targetWeek2json.includes(week4), isTrue);
    expect(targetWeek2json.includes(week5), isTrue);

    // Test first last functions
    expect(targetWeek1.first(), isNull);
    expect(targetWeek1.last(), isNull);
    expect(targetWeek1json.first(), isNull);
    expect(targetWeek1json.last(), isNull);
  });


  // Test for the TE: TEIsAfter.dates() method
  //   TODO: This test group could use some more meat.
  test('dates() method', () {
    DateTime base   = new DateTime.utc(2013, 7, 11, 0, 0, 0, -30);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 1);

    TEIsAfter targetDate = new TEIsAfter.DateTime(target);
    TEIsAfter targetDatejson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(targetDate)));
    TEIsAfter targetMonth = new TEIsAfter.Month(8);
    TEIsAfter targetMonthjson = new TEIsAfter.fromJson(JSON.decode(JSON.encode(targetMonth)));

    List<DateTime> results;
    targetDate.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
      expect(results[0], target);
      expect(results[9], target.add(new Duration(milliseconds:9)));
    });

    targetDatejson.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
      expect(results[0], target);
      expect(results[9], target.add(new Duration(milliseconds:9)));
    });

    // Test the result count limiting parameter
    targetMonth.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });

    // Test the iteration limiting parameter
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
  });
  });
}