library testisbefore;
import "package:unittest/unittest.dart";
import "package:dartemporal/dartemporal.dart";
import "dart:convert";

testIsBefore() {
  group('TEIsBefore -> ', () {

    // This group of tests checks just the type() methods of each of the TE instantiators
    //   The values in the constructors do not really matter, they are just to get an object
    //   to test against.
    test('type() methods', () {
      expect(new TEIsBefore.Year(2012).type(), TExpression.YEAR);
      expect(new TEIsBefore.Month(3).type(), TExpression.MONTH);
      expect(new TEIsBefore.DateTime(new DateTime.now()).type(), TExpression.DATETIME);
      expect(new TEIsBefore.WeekOfYear(35).type(), TExpression.WEEKOFYEAR);
      expect(new TEIsBefore.DayOfWeek(3).type(), TExpression.DAYOFWEEK);
      expect(new TEIsBefore.DayOfMonth(22).type(), TExpression.DAYOFMONTH);
      expect(new TEIsBefore.DayOfYear(123).type(), TExpression.DAYOFYEAR);
      expect(new TEIsBefore.Hour(22).type(), TExpression.HOUR);
      expect(new TEIsBefore.Minute(33).type(), TExpression.MINUTE);
      expect(new TEIsBefore.Second(45).type(), TExpression.SECOND);
      expect(new TEIsBefore.Millisecond(333).type(), TExpression.MILLISECOND);
      expect(new TEIsBefore.Date(new DateTime.now()).type(), TExpression.DATE);
      expect(new TEIsBefore.WeekOfMonth(-2).type(), TExpression.WEEKOFMONTH);
    });

    // Tests for the TEIsBefore.Year() constructor
    test('Year() constructor', () {
      DateTime testDate;

      // We will test times every 23 1/3 hours one thousand times
      //  This lets us hit a whole bunch of dates/times
      [-2000, 0, 101, 2012, 5313].forEach((y){
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Year(y);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Year(y, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);
        DateTime yearEnd = new DateTime.utc(y+1,1,1,0,0,0,-1);
        DateTime yearEndni = new DateTime.utc(y,1,1,0,0,0,-1);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTEjson.last(), yearEnd);
        expect(testTEni.first(), isNull);
        expect(testTEnijson.last(), yearEndni);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.year <= y);
          expect(testTEjson.includes(testDate), testDate.year <= y);
          expect(testTEni.includes(testDate), testDate.year < y);
          expect(testTEnijson.includes(testDate), testDate.year < y);
        }
      });
    });

  // Test for the TE: TEIsBefore.Month() constructor
  test('Month() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of various years
    [-2000, 0, 101, 2012, 5313].forEach((y){
      // test for each month of the year
      for(int m=1; m<13; m++) {
        /// We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Month(m);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Month(m, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.month <= m);
          expect(testTEjson.includes(testDate), testDate.month <= m);
          expect(testTEni.includes(testDate), testDate.month < m);
          expect(testTEnijson.includes(testDate), testDate.month < m);
        }
      }
    });
  });

  // Test for the TE: Is.DayOfWeek() constructor
  test('DayOfWeek() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dow=0; dow<7; dow++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.DayOfWeek(dow);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.DayOfWeek(dow, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.weekday <= dow);
          expect(testTEjson.includes(testDate), testDate.weekday <= dow);
          expect(testTEni.includes(testDate), testDate.weekday < dow);
          expect(testTEnijson.includes(testDate), testDate.weekday < dow);
        }
      }
    });
  });

  // Test for the TE: TEIsBefore.DayOfMonth() constructor
  test('DayOfMonth() constructor', () {
    DateTime testDate;

    // We will test times every 23 1/3 hours one thousand times
    //  This lets us hit a whole bunch of dates/times
    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int dom=1; dom<32; dom++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.DayOfMonth(dom);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.DayOfMonth(dom, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y - 1, 10, 31, 0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(minutes:1400 * i));
          expect(testTE.includes(testDate), testDate.day <= dom);
          expect(testTEjson.includes(testDate), testDate.day <= dom);
          expect(testTEni.includes(testDate), testDate.day < dom);
          expect(testTEnijson.includes(testDate), testDate.day < dom);
        }
      }
    });
  });

  // Test for the TE: TEIsBefore.DayOfYear() constructor
  test('DayOfYear() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int doy=1; doy<366; doy++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.DayOfYear(doy);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.DayOfYear(doy, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y,1,1,0,0,0,0);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        int daycounter = 1;

        // Test inclusivity works
        for(int i=0; i < 365; i++) {
          testDate = base.add(new Duration(hours: 24 * i));
          expect(testTE.includes(testDate), daycounter <= doy);
          expect(testTEjson.includes(testDate), daycounter <= doy);
          expect(testTEni.includes(testDate), daycounter < doy);
          expect(testTEnijson.includes(testDate), daycounter < doy);
          daycounter += 1;
        }
      }
    });
  });

  // Test for the TE: TEIsBefore.WeekOfYear() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfYear() constructor', () {
    DateTime week25  = new DateTime(2013, 6, 19, 0, 0, 0, 0);
    DateTime week24  = new DateTime(2013, 6, 16, 0, 0, 0, 0);
    DateTime week26  = new DateTime(2013, 6, 24, 0, 0, 0, 0);

    TEIsBefore onWeek25 = new TEIsBefore.WeekOfYear(25);
    TEIsBefore onWeek25json = new TEIsBefore.fromJson(JSON.decode(JSON.encode(onWeek25)));
    TEIsBefore onWeek25ni = new TEIsBefore.WeekOfYear(25,false);
    TEIsBefore onWeek25nijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(onWeek25ni)));

    // Test inclusivity works
    expect(onWeek25.includes(week24), isTrue);
    expect(onWeek25.includes(week25), isTrue);
    expect(onWeek25.includes(week26), isFalse);
    expect(onWeek25json.includes(week24), isTrue);
    expect(onWeek25json.includes(week25), isTrue);
    expect(onWeek25json.includes(week26), isFalse);
    expect(onWeek25ni.includes(week24), isTrue);
    expect(onWeek25ni.includes(week25), isFalse);
    expect(onWeek25ni.includes(week26), isFalse);
    expect(onWeek25nijson.includes(week24), isTrue);
    expect(onWeek25nijson.includes(week25), isFalse);
    expect(onWeek25nijson.includes(week26), isFalse);

    // Test first last functions
    expect(onWeek25.first(), isNull);
    expect(onWeek25.last(), isNull);
    expect(onWeek25json.first(), isNull);
    expect(onWeek25json.last(), isNull);
    expect(onWeek25ni.first(), isNull);
    expect(onWeek25ni.last(), isNull);
    expect(onWeek25nijson.first(), isNull);
    expect(onWeek25nijson.last(), isNull);
  });

  // Tests for the TEIsBefore.Hour() constructor
  test('Hour() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int h=0; h<24; h++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Hour(h);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Hour(h, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.hour <= h);
          expect(testTEjson.includes(testDate), testDate.hour <= h);
          expect(testTEni.includes(testDate), testDate.hour < h);
          expect(testTEnijson.includes(testDate), testDate.hour < h);
        }
      }
    });
  });

  // Tests for the TEIsBefore.Minute() constructor
  test('Minute() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int m=0; m<60; m++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Minute(m);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Minute(m, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.minute <= m);
          expect(testTEjson.includes(testDate), testDate.minute <= m);
          expect(testTEni.includes(testDate), testDate.minute < m);
          expect(testTEnijson.includes(testDate), testDate.minute < m);
        }
      }
    });
  });

  // Tests for the TEIsBefore.Second() constructor
  test('Second() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int s=0; s<60; s++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Second(s);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Second(s, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(seconds:57 * i));
          expect(testTE.includes(testDate), testDate.second <= s);
          expect(testTEjson.includes(testDate), testDate.second <= s);
          expect(testTEni.includes(testDate), testDate.second < s);
          expect(testTEnijson.includes(testDate), testDate.second < s);
        }
      }
    });
  });

  // Tests for the TEIsBefore.Millisecond() constructor
  test('Millisecond() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        TEIsBefore testTE = new TEIsBefore.Millisecond(ms);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Millisecond(ms, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,5);

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), isNull);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), isNull);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), isNull);
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), isNull);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1007 * i));
          expect(testTE.includes(testDate), testDate.millisecond <= ms);
          expect(testTEjson.includes(testDate), testDate.millisecond <= ms);
          expect(testTEni.includes(testDate), testDate.millisecond < ms);
          expect(testTEnijson.includes(testDate), testDate.millisecond < ms);
        }
      }
    });
  });

  // Tests for the TEIsBefore.DateTime() constructor
  test('DateTime() constructor', () {
    DateTime testDate;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIsBefore testTE = new TEIsBefore.DateTime(base);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.DateTime(base, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        // Test first last functions
        expect(testTE.first(), isNull);
        expect(testTE.last(), base);
        expect(testTEjson.first(), isNull);
        expect(testTEjson.last(), base);
        expect(testTEni.first(), isNull);
        expect(testTEni.last(), base.add(new Duration(milliseconds: -1)));
        expect(testTEnijson.first(), isNull);
        expect(testTEnijson.last(), base.add(new Duration(milliseconds: -1)));

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(milliseconds:1 * i));
          expect(testTE.includes(testDate), testDate.millisecondsSinceEpoch <= base.millisecondsSinceEpoch);
          expect(testTEjson.includes(testDate), testDate.millisecondsSinceEpoch <= base.millisecondsSinceEpoch);
          expect(testTEni.includes(testDate), testDate.millisecondsSinceEpoch < base.millisecondsSinceEpoch);
          expect(testTEnijson.includes(testDate), testDate.millisecondsSinceEpoch < base.millisecondsSinceEpoch);
        }
      }
    });
  });

  // Tests for the TEIsBefore.Date() constructor
  test('Date() constructor', () {
    DateTime testDate;
    DateTime dayStart;
    DateTime dayEnd;
    DateTime dayStartni;
    DateTime dayEndni;

    [-2000, 0, 101, 2012, 5313].forEach((y){
      for(int ms=0; ms<1000; ms++) {
        // We create these two TE's which should be in effect identical but are constructed differently
        DateTime base = new DateTime.utc(y, 10, 31, 2,41,14,543);
        TEIsBefore testTE = new TEIsBefore.Date(base);
        TEIsBefore testTEjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTE)));

        // And these two TE's for non-inclusive which should be in effect identical but are constructed differently
        TEIsBefore testTEni = new TEIsBefore.Date(base, false);
        TEIsBefore testTEnijson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(testTEni)));

        dayEnd = new DateTime.utc(base.year, base.month, base.day+1, 0,0,0,-1);
        dayEndni = new DateTime.utc(base.year, base.month, base.day, 0,0,0,-1);

        // Test inclusivity works
        for(int i=0; i < 1000; i++) {
          testDate = base.add(new Duration(hours:95 * i));

          expect(testTE.includes(testDate), testDate.millisecondsSinceEpoch <= base.millisecondsSinceEpoch);
          expect(testTEjson.includes(testDate), testDate.millisecondsSinceEpoch <= base.millisecondsSinceEpoch);

          expect(testTE.first(), isNull);
          expect(testTEjson.first(), isNull);
          expect(testTE.last(), dayEnd);
          expect(testTEjson.last(), dayEnd);

          expect(testTEni.includes(testDate), testDate.millisecondsSinceEpoch < base.millisecondsSinceEpoch);
          expect(testTEnijson.includes(testDate), testDate.millisecondsSinceEpoch < base.millisecondsSinceEpoch);

          expect(testTEni.first(), isNull);
          expect(testTEnijson.first(), isNull);
          expect(testTEni.last(), dayEndni);
          expect(testTEnijson.last(), dayEndni);
        }
      }
    });
  });

  // Test for the TE: TEIsBefore.WeekOfMonth() constructor
  //   TODO: This test group could use some more meat.
  test('WeekOfMonth() constructor', () {
    DateTime week1 = new DateTime.utc(2013, 7,  3, 0, 0, 0, 0);  // Week 1 of 5
    DateTime week2 = new DateTime.utc(2013, 7, 10, 0, 0, 0, 0); // Week 2 of 5
    DateTime week3  = new DateTime.utc(2013, 7, 17, 0, 0, 0, 0); // Week 3 of 5
    DateTime week4 = new DateTime.utc(2013, 7, 24, 0, 0, 0, 0); // Week 4 of 5
    DateTime week5  = new DateTime.utc(2013, 7, 31, 0, 0, 0, 0); // Week 5 of 5

    TEIsBefore targetWeek0 = new TEIsBefore.WeekOfMonth(0);
    TEIsBefore targetWeek1 = new TEIsBefore.WeekOfMonth(3);
    TEIsBefore targetWeek2 = new TEIsBefore.WeekOfMonth(-2);
    TEIsBefore targetWeek0json = new TEIsBefore.fromJson(JSON.decode(JSON.encode(targetWeek0)));
    TEIsBefore targetWeek1json = new TEIsBefore.fromJson(JSON.decode(JSON.encode(targetWeek1)));
    TEIsBefore targetWeek2json = new TEIsBefore.fromJson(JSON.decode(JSON.encode(targetWeek2)));

    // Test inclusivity works
    expect(targetWeek0.includes(week1), isFalse);
    expect(targetWeek0.includes(week2), isFalse);
    expect(targetWeek0.includes(week3), isFalse);
    expect(targetWeek0.includes(week4), isFalse);
    expect(targetWeek0.includes(week5), isFalse);

    expect(targetWeek1.includes(week1), isTrue);
    expect(targetWeek1.includes(week2), isTrue);
    expect(targetWeek1.includes(week3), isTrue);
    expect(targetWeek1.includes(week4), isFalse);
    expect(targetWeek1.includes(week5), isFalse);

    expect(targetWeek2.includes(week1), isTrue);
    expect(targetWeek2.includes(week2), isTrue);
    expect(targetWeek2.includes(week3), isTrue);
    expect(targetWeek2.includes(week4), isTrue);
    expect(targetWeek2.includes(week5), isFalse);

    expect(targetWeek0json.includes(week1), isFalse);
    expect(targetWeek0json.includes(week2), isFalse);
    expect(targetWeek0json.includes(week3), isFalse);
    expect(targetWeek0json.includes(week4), isFalse);
    expect(targetWeek0json.includes(week5), isFalse);

    expect(targetWeek1json.includes(week1), isTrue);
    expect(targetWeek1json.includes(week2), isTrue);
    expect(targetWeek1json.includes(week3), isTrue);
    expect(targetWeek1json.includes(week4), isFalse);
    expect(targetWeek1json.includes(week5), isFalse);

    expect(targetWeek2json.includes(week1), isTrue);
    expect(targetWeek2json.includes(week2), isTrue);
    expect(targetWeek2json.includes(week3), isTrue);
    expect(targetWeek2json.includes(week4), isTrue);
    expect(targetWeek2json.includes(week5), isFalse);

    // Test first last functions
    expect(targetWeek1.first(), isNull);
    expect(targetWeek1.last(), isNull);
    expect(targetWeek1json.first(), isNull);
    expect(targetWeek1json.last(), isNull);
  });


  // Test for the TE: TEIsBefore.dates() method
  //   TODO: This test group could use some more meat.
  test('dates() method', () {
    DateTime base   = new DateTime.utc(2013, 7, 11, 0, 0, 0, -30);
    DateTime target = new DateTime.utc(2013, 7, 11, 0, 0, 0, 1);

    TEIsBefore targetDate = new TEIsBefore.DateTime(target);
    TEIsBefore targetDatejson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(targetDate)));
    TEIsBefore targetMonth = new TEIsBefore.Month(8);
    TEIsBefore targetMonthjson = new TEIsBefore.fromJson(JSON.decode(JSON.encode(targetMonth)));

    List<DateTime> results;
    targetDate.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
      expect(results[0], base);
      expect(results[9], base.add(new Duration(milliseconds:9)));
    });

    targetDatejson.dates(base, new Duration(milliseconds: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
      expect(results[0], base);
      expect(results[9], base.add(new Duration(milliseconds:9)));
    });

    // Test the result count limiting parameter
    targetMonth.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonth.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1)).then((List<DateTime> results) {
      expect(results.length == 10, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 20).then((List<DateTime> results) {
      expect(results.length == 20, isTrue);
    });
    targetMonthjson.dates(base, new Duration(days: 1), 5).then((List<DateTime> results) {
      expect(results.length == 5, isTrue);
    });

    // Test the iteration limiting parameter
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonth.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000).then((List<DateTime> results) {
      expect(results.length == 10000, isTrue);
    });
    targetMonthjson.dates(new DateTime.utc(2013,8,2,0,0,0,0), new Duration(seconds: 1), 20000, 20000).then((List<DateTime> results) {
      expect(results.length == 20000, isTrue);
    });
  });
  });
}